from ams_instrument.ams_instrument import ams_instrument
from abc import ABC, abstractmethod

class ams_ppmu_instrument(ams_instrument):
    """
    This interface defines the functionality of an PPMU instrument
    """

    def __init__(self, device_dict: dict) -> object:
        """__init__ 
        Initializes the ams_PPMU_instrument object

        Args:
            device_dict (dict): dictionary of instrument settings

        Returns:
            object: ams_PPMU_instrument object
        """
        pass


    @abstractmethod
    def source_voltage(
        self, 
        voltage_level: float, 
        current_limit: float, 
        channel_name: str,
        voltage_range: float=None,
        current_range: float=None
    ):
        """source_voltage 
        This method sources out voltage to a certain channel with a defined current limit

        Args:
            voltage_level (float): PPMU terminal voltage
            current_limit (float): PPMU current limit
            channel_name (str): Name of the channel on PPMU in use
            voltage_range (float, optional): voltage range - if not given, no new limit is set. Defaults to None.
            current_range (float, optional): current range - if not given, no new limit is set. Defaults to None.
        """
        pass


    @abstractmethod
    def source_current(
        self, 
        current_level: float, 
        voltage_limit: float, 
        channel_name: str,
        voltage_range: float=None,
        current_range: float=None
    ):
        """source_current 
        This method sources out current to a certain channel with a defined voltage limit

        Args:
            current_level (float): PPMU terminal current
            voltage_limit (float): PPMU voltage limit
            channel_name (str): Name of the channel on PPMU in use
            voltage_range (float, optional): voltage range - if not given, no new limit is set. Defaults to None.
            current_range (float, optional): current range - if not given, no new limit is set. Defaults to None.
        """
        pass


    @abstractmethod
    def measure_voltage(
        self, 
        channel_name: str,
        range: str=None, 
    ) -> float:
        """measure_voltage 
        This method measures the voltage on a certain channel with given parameters

        Args:
            channel_name (str): PPMU channel name
            range (str, optional): measurement range. Defaults to None.

        Returns:
            float: measured voltage
        """
        pass


    @abstractmethod
    def measure_current(
        self, 
        channel_name: str,
        range: str=None, 
    ) -> float:
        """
        This method measures the current on a certain channel with given parameters

        Args:
            channel_name (str): PPMU channel name
            range (str, optional): measurement range. Defaults to None.

        Returns:
            float: measured current
        """
        pass


    @abstractmethod
    def stop_sourcing(
        self,
        channel_name: str
    ):
        """stop_sourcing 
        This method stops sourcing on the given terminal

        Args:
            channel_name (str): name of the channel
        """
        pass


    @abstractmethod
    def ppmu_terminal_enable(
        self,
        channel_name: str,
        enable: bool=False
    ):
        """ppmu_terminal_enable 
        This method sets the state of the terminal of PPMU

        Args:
            channel_name (str): name of the PPMU channel
            enable (bool, optional): set this true to activate. Defaults to False.
        """


    @abstractmethod
    def ppmu_measure_frequency(
        self,
        channel_name: str,
        hysteresis_enabled: bool = True,
        measurement_time: float = None
    ) -> float:
        """
        This method measures the frequency of a periodic signal applied to a pin

        Args:
            channel_name (str): name of the pin to measure frequency on
            hysteresis_enabled (bool): Choose wether to use a hysteresis on the comparator before the frequency counter or not (defaults to True)
            measurement_time (float): set the time the frequency counter counts. If ommitted, the measurement time is not changed (defaults to None)

        Returns:
            float: The frequency measured in Hz
        """
        pass


    @abstractmethod
    def ppmu_generate_clock(
        self,
        channel_name: str,
        frequency: float,
        level_high: float,
        level_low: float=0,
    ):
        """
        This method generates a clock signal to a given a pin

        Args:
            channel_name (str): name of the pin to use
            frequency (float): frequency of the clock signal in Hz
            level_high (float): voltage level when the clock is high in V
            level_low (float, optional): voltage level when the clock is low in V. Defaults to 0.
        """
        pass


    @abstractmethod
    def ppmu_stop_clock_generation(
        self,
        channel_name: str
    ):
        """
        This method stop the clock signal generation at a given pin

        Args:
            channel_name (str): name of the channel
        """
        pass


    @abstractmethod
    def load_scan_pattern(
        self,
        pattern_file_path: str,
        timing_file_path: str,
        signal_level_file_path: str,
        pin_map_file_path: str=None
    ):
        """
        This method uploads a scan pattern file to the ppmu.
        This method is implemented only to prepare the scan patterns right before bursting them out to the chip.
        If any kind of communication is intenteded to use after this method is called, first all the communication channels have to be reconfigured.

        Giving the pin map file is optional and highly UN-RECOMMENDED !!!!.

        Args:
            pattern_file_path (str): path to the pattern file
            timing_file_path (str): path to the file that defines the timing behaviour of the different pins/signals
            signal_level_file_path (str): path to the file that defines the voltage levels for the different pins
            pin_map_file_path (str, optional): path to the file that defines the pins. Defaults to None.
        """
        pass


    @abstractmethod
    def bust_scan_pattern(
        self,
        pattern_name: str,
        timeout: int=10
    ) -> dict:
        """
        This method busts out the selected pattern that is previously been loaded to the pattern instrument.

        Args:
            pattern_name (str): name of the pattern to be bursted
            timeout (int, optional): timeout in ms. Defaults to 10.

        Returns:
            dict: dictionary about the results
        """
        pass