from datetime import time
from ams_ppmu import ams_ppmu_instrument
import logging, nidigital
from enum import Enum

my_logger = logging.getLogger(__name__)


class ams_PXI_PPMU(ams_ppmu_instrument):
    """
    This interface defines the functionality of an PPMU instrument
    """

    class PPMU_current_range(Enum):
        micro_2 = 2e-6
        micro_32 = 32e-6
        micro_128 = 128e-6
        mili_2 = 2e-3
        mili_32 = 32e-3


    def __init__(self, device_dict: dict) -> object:
        """__init__ 
        Initializes the ams_PPMU_instrument object
        If the instance field is included into the dictionary then the class will use it as instrument reference to the HW session

        Args:
            device_dict (dict): dictionary of instrument settings

        Returns:
            object: ams_PPMU_instrument object
        """
        self.settings_dict = device_dict
        self.pin_map_file = device_dict['Details']['ni_pin_map_file']
        if 'instance' not in self.settings_dict['Details'].keys():
            my_logger.debug('Initialize new session')
            self.handler = nidigital.Session(
                resource_name=self.settings_dict['Details']['max_alias'],
                reset_device=False
            )
        else:
            my_logger.debug('Session is injected')
            self.handler = self.settings_dict['Details']['instance']


    def source_voltage(
        self, 
        voltage_level: float, 
        current_limit: float, 
        channel_name: str,
        voltage_range: float=None,
        current_range: float=None
    ):
        """source_voltage 
        This method sources out voltage to a certain channel with a defined current limit

        Args:
            voltage_level (float): PPMU terminal voltage
            current_limit (float): PPMU current limit
            channel_name (str): Name of the channel on PPMU in use
            voltage_range (float, optional): voltage range - if not given, no new limit is set. Defaults to None.
            current_range (float, optional): current range - if not given, no new limit is set. Defaults to None.
        """
        self.handler.channels[channel_name].ppmu_output_function = nidigital.PPMUOutputFunction.VOLTAGE
        self.handler.channels[channel_name].ppmu_voltage_level = voltage_level
        self.handler.channels[channel_name].ppmu_current_limit_range = current_limit
        if current_range != None:
            self.handler.channels[channel_name].ppmu_current_level_range = current_range
        self.handler.channels[channel_name].ppmu_source()


    def source_current(
        self, 
        current_level: float, 
        voltage_limit: float, 
        channel_name: str,
        voltage_range: float=None,
        current_range: float=None
    ):
        """source_current 
        This method sources out current to a certain channel with a defined voltage limit

        Args:
            current_level (float): PPMU terminal current
            voltage_limit (float): PPMU voltage limit
            channel_name (str): Name of the channel on PPMU in use
            voltage_range (float, optional): voltage range - if not given, no new limit is set. Defaults to None.
            current_range (float, optional): current range - if not given, no new limit is set. Defaults to None.
        """
        self.handler.channels[channel_name].ppmu_output_function = nidigital.PPMUOutputFunction.CURRENT
        self.handler.channels[channel_name].ppmu_current_level = current_level
        self.handler.channels[channel_name].ppmu_voltage_limit_high = voltage_limit
        if current_range != None:
            self.handler.channels[channel_name].ppmu_current_level_range = current_range
        self.handler.channels[channel_name].ppmu_source()


    def measure_voltage(
        self, 
        channel_name: str,
        range: str=None, 
    ) -> float:
        """measure_voltage 
        This method measures the voltage on a certain channel with given parameters

        Args:
            channel_name (str): PPMU channel name
            range (str, optional): measurement range. Defaults to None.

        Returns:
            float: measured voltage
        """
        measurement = self.handler.channels[channel_name].ppmu_measure(nidigital.PPMUMeasurementType.VOLTAGE)
        my_logger.debug(measurement)
        return measurement[0]


    def measure_current(
        self, 
        channel_name: str,
        range: str=None, 
    ) -> float:
        """
        This method measures the current on a certain channel with given parameters

        Args:
            channel_name (str): PPMU channel name
            range (str, optional): measurement range. Defaults to None.

        Returns:
            float: measured current
        """
        measurement = self.handler.channels[channel_name].ppmu_measure(nidigital.PPMUMeasurementType.CURRENT)
        my_logger.debug(measurement)
        return measurement[0]


    def stop_sourcing(
        self,
        channel_name: str
    ):
        """stop_sourcing 
        This method stops sourcing on the given terminal

        Args:
            channel_name (str): name of the channel
        """
        if self.handler.channels[channel_name].ppmu_output_function == nidigital.PPMUOutputFunction.VOLTAGE:
            self.handler.channels[channel_name].ppmu_voltage_level = 0
        else:
            self.handler.channels[channel_name].ppmu_current_level = 0
        self.handler.channels[channel_name].ppmu_source()


    def GetInstrumentError(self) -> str:
        """IdentifyInstrument 
        Gets the identifier of the instrument as string

        Returns:
            ID_string [String]: The identification string from the instrument
        """
        return self.handler._get_error()


    def IdentifyInstrument(self) -> str:
        """GetInstrumentError
        This function returns the error string from the instrument

        Returns:
            Error_string [String]: The error string from the instrument
        """
        return self.handler.logical_name


    def ResetInstrumehnt(self) -> str:
        """ResetInstrumehnt 
        This function resets the instrument
        """
        return self.handler.reset()


    def ppmu_terminal_enable(
        self,
        channel_name: str,
        enable: bool=False
    ):
        """ppmu_terminal_enable 
        This method sets the state of the terminal of PPMU

        Args:
            channel_name (str): name of the PPMU channel
            enable (bool, optional): set this true to activate. Defaults to False.
        """
        if enable:
            self.handler.channels[channel_name].selected_function = nidigital.SelectedFunction.PPMU
        else:
            self.handler.channels[channel_name].selected_function = nidigital.SelectedFunction.DISCONNECT
        

    def ppmu_measure_frequency(
        self,
        channel_name: str,
        measurement_time: float = None
    ) -> float:
        """
        This method measures the frequency of a periodic signal applied to a pin

        Args:
            channel_name (str): name of the pin to measure frequency on
            hysteresis_enabled (bool): Choose wether to use a hysteresis on the comparator before the frequency counter or not (defaults to True)
            measurement_time (float): set the time the frequency counter counts. If ommitted, the measurement time is not changed (defaults to None)

        Returns:
            float: The frequency measured in Hz
        """
        #make sure the pin is a digital pin
        self.handler.channels[channel_name].selected_function = nidigital.SelectedFunction.DIGITAL
        if measurement_time:
            self.handler.channels[channel_name].frequency_counter_measurement_time = measurement_time
        
        return self.handler.channels[channel_name].frequency_counter_measure_frequency()[0]

    def ppmu_generate_clock(
        self,
        channel_name: str,
        frequency: float,
        level_high: float,
        level_low: float=0
    ):
        """
        This method generates a clock signal to a given a pin

        Args:
            channel_name (str): name of the pin to use
            frequency (float): frequency of the clock signal in Hz
            level_high (float): voltage level when the clock is high in V
            level_low (float, optional): voltage level when the clock is low in V. Defaults to 0.
        """
        self.handler.channels[channel_name].clock_generator_frequency = frequency
        self.handler.channels[channel_name].configure_voltage_levels(
            vil=level_low,
            vih=level_high,
            vol=level_low + 0.3,
            voh=level_high - 0.3,
            vterm=0
        )
        #self.handler.channels[channel_name].configure_active_load_levels(level_low, level_high, 0)
        self.handler.channels[channel_name].clock_generator_generate_clock(frequency)


    def ppmu_stop_clock_generation(
        self,
        channel_name: str
    ):
        """
        This method stop the clock signal generation at a given pin

        Args:
            channel_name (str): name of the channel
        """
        self.handler.channels[channel_name].clock_generator_abort()


    def load_scan_pattern(
        self,
        pattern_file_path: str,
        timing_file_path: str,
        signal_level_file_path: str,
        pin_map_file_path: str=None
    ):
        """
        This method uploads a scan pattern file to the ppmu.
        This method is implemented only to prepare the scan patterns right before bursting them out to the chip.
        If any kind of communication is intenteded to use after this method is called, first all the communication channels have to be reconfigured.

        Giving the pin map file is optional and highly UN-RECOMMENDED !!!!.

        Args:
            pattern_file_path (str): path to the pattern file
            timing_file_path (str): path to the file that defines the timing behaviour of the different pins/signals
            signal_level_file_path (str): path to the file that defines the voltage levels for the different pins
            pin_map_file_path (str, optional): path to the file that defines the pins. Defaults to None.
        """
        self.handler.load_pattern(pattern_file_path)
        self.handler.load_specifications_levels_and_timing(
            levels_file_paths=signal_level_file_path,
            timing_file_paths=timing_file_path
        )
        if pin_map_file_path != None:
            self.handler.load_pin_map(pin_map_file_path)


    def bust_scan_pattern(
        self,
        pattern_name: str,
        timeout: int=10
    ) -> dict:
        """
        This method busts out the selected pattern that is previously been loaded to the pattern instrument.

        Args:
            pattern_name (str): name of the pattern to be bursted
            timeout (int, optional): timeout in ms. Defaults to 10.

        Returns:
            dict: dictionary about the results
        """
        return self.handler.burst_pattern(pattern_name, timeout=timeout)